<?php

namespace Good\Manners;

abstract class Storage
{
    private $validationToken;

    protected $storableFactory;
    private $defaultStorableFactory;

    public function __construct()
    {
        $this->validationToken = new ValidationToken();

        $this->defaultStorableFactory = new DefaultStorableFactory();
        $this->storableFactory = $this->defaultStorableFactory;
    }

    public function __destruct()
    {
        $this->flush();
    }

    public function setStorableFactory(StorableFactory $factory)
    {
        $this->storableFactory = $factory;
    }

    public function registerType($parentType, $childType)
    {
        if ($this->storableFactory != $this->defaultStorableFactory)
        {
            throw new Exception("Tried to register a type to the StorableFactory after " .
                                    "it was exchanged set to something else than the default.");
        }

        $this->defaultStorableFactory->registerType($parentType, $childType);
    }

    public function createEmptyStorable($type)
    {
        return $this->storableFactory->createStorable($type);
    }

    protected function invalidate()
    {
        $this->validationToken->invalidate();
        $this->validationToken = new ValidationToken();
    }

    abstract public function insert(Storable $storable);
    abstract public function modifyAny(Condition $condition, Storable $modifications);
    abstract public function fetchAll($condition, Resolver $resolver = null, Page $page = null);

    abstract public function resolve(Storable $storable, Resolver $resolver = null);
    abstract public function resolveCollection(ResolvableCollection $storable, Resolver $resolver = null);

    abstract public function flush();
}

?>
